# cox regression
# p-value is obtained from a likelihood ratio test
# eg: --my-R=etc/myR.coxph.R --R-par="SURV_FILE:/path/to/follow_up_data_and_covariates;SURV_TIME:FU,Death;SURV_COV:+AgeDx+Tstage"
#     If you don't want covariates, you still need to set "SURV_COV:".
# vAAA will find SURV_FILE and determine SURV_COLS (1st column "character", others "numeric"). Otherwise, you can set SURV_COLS.
# VERSION: 2022-12-21. Ignore invariable covariates.

options(warn=-1)
suppressWarnings(library("survival"))

ONE_SIDED <- one_sided
COV_STR <- "SURV_COV" # "+Sex+AgeDx+AdvStage+Site+Histology"
y<-read.table("SURV_FILE", header=TRUE, colClasses=c(SURV_COLS))
merged <- merge(x, y, by="SeqID")
final_data <- na.omit(merged)

THIS_STR <- COV_STR
covars <- unlist(strsplit(THIS_STR, "\\+"))
covars <- covars[covars != ""]
for (j in covars){
  if (length(unique(get(final_data)[[j]]))==1){
    THIS_STR <- gsub(paste0("\\+", j), "", THIS_STR)
  }
}
f <- paste0("Surv(SURV_TIME) ~ gene + COVARIATE_NAMES", THIS_STR)
g <- paste0("Surv(SURV_TIME) ~ COVARIATE_NAMES", THIS_STR)

res.cox.full <- coxph(as.formula(f), data=final_data, weights=IndWt)
res.cox.full.sum <- summary(res.cox.full)
res.cox.red <- coxph(as.formula(g), data=final_data, weights=IndWt)
res.cox.red.sum <- summary(res.cox.red)
beta <- res.cox.full.sum$coef[1]
pv <- pchisq(abs(res.cox.full.sum$loglik[2]-res.cox.red.sum$loglik[2])*2,1,lower.tail=FALSE)
if (ONE_SIDED) {
  pv <- ifelse(beta > 0, pv/2, (1-(pv/2)))
}
cat("MyR_pval ", pv, "\n", sep = "")
hr <- res.cox.full.sum$conf.int[1,1]
lower <- res.cox.full.sum$conf.int[1,3]
upper <- res.cox.full.sum$conf.int[1,4]
cat("MyR_info HR=", hr, "[", lower, ",", upper, "]\n", sep = "")
