# Firth's cox regression
# coxphf does not support individual weight.
# Follow up time must not be <=0; otherwise coxphf will give an error: "NA/NaN/Inf in foreign function call (arg 3)".
# VERSION: 2022-12-21. Ignore invariable covariates.

options(warn=-1)
suppressWarnings(library("survival"))
suppressWarnings(library("coxphf"))

ONE_SIDED <- one_sided
COV_STR <- "SURV_COV" # "+Sex+AgeDx+AdvStage+Site+Histology"
y<-read.table("SURV_FILE", header=TRUE, colClasses=c(SURV_COLS))
merged <- merge(x, y, by="SeqID")
final_data <- na.omit(merged)

THIS_STR <- COV_STR
covars <- unlist(strsplit(THIS_STR, "\\+"))
covars <- covars[covars != ""]
for (j in covars){
  if (length(unique(get(final_data)[[j]]))==1){
    THIS_STR <- gsub(paste0("\\+", j), "", THIS_STR)
  }
}
f <- paste0("Surv(SURV_TIME) ~ gene + COVARIATE_NAMES + RAND_EFFECT_GRP", THIS_STR)

res.cox.full <- coxphf(as.formula(f), data=final_data)
res.cox.full.sum <- summary(res.cox.full)
beta <- res.cox.full.sum$coef[1]
pv <- res.cox.full.sum$prob[1]
if (ONE_SIDED) {
  pv <- ifelse(beta > 0, pv/2, (1-(pv/2)))
}
hr <- exp(beta)
lower <- res.cox.full.sum$ci.lower[1]
upper <- res.cox.full.sum$ci.upper[1]
cat("MyR_beta ", beta, "\n", sep = "")
cat("MyR_pval ", pv, "\n", sep = "")
cat("MyR_info HR=", hr, "[", lower, ",", upper, "]\n", sep = "")
